﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using System.Net.Http.Formatting;
using System.Net.Http.Headers;
using DataLayerEF.Entities;
using System.IO;
using System.Threading.Tasks;

namespace AW3
{
	public class CategoryFormatter : MediaTypeFormatter
	{
		public CategoryFormatter()
		{
			this.SupportedMediaTypes.Add(new MediaTypeHeaderValue("image/png"));
			this.SupportedMediaTypes.Add(new MediaTypeHeaderValue("image/jpeg"));
			this.SupportedMediaTypes.Add(new MediaTypeHeaderValue("text/csv"));
			this.SupportedMediaTypes.Add(new MediaTypeHeaderValue("text/plain"));
			this.SupportedMediaTypes.Add(new MediaTypeHeaderValue("text/html"));
		}

		protected override bool CanReadType(Type type)
		{
			return false;
		}

		protected override Task<object> OnReadFromStreamAsync(Type type, Stream stream, HttpContentHeaders contentHeaders, FormatterContext formatterContext)
		{
			throw new NotImplementedException();
		}

		protected override bool CanWriteType(Type type)
		{
			return true;
		}

		protected override Task OnWriteToStreamAsync(Type type, object value, Stream stream, HttpContentHeaders contentHeaders, FormatterContext formatterContext, System.Net.TransportContext transportContext)
		{
			var category = value as Category;
			if(category == null || category.Picture == null)
				return new Task(() => { });

			var t = new Task(() =>
			{
				var isThumb = false;
				switch(contentHeaders.ContentType.MediaType)
				{
					case "image/jpeg":
						{
							var blobPicture = category.Picture;
							stream.Write(blobPicture, 0, blobPicture.Length);
						}
						break;
					case "image/png":
						{
							var blobPicture = category.Picture;
							Helpers.BlobToStream<bool>(blobPicture, st =>
							{
								using(MemoryStream ms = new MemoryStream())
								{
									var size = Helpers.GetImageSizeCm(st);
									st.Seek(0, SeekOrigin.Begin);
									int w = 0, h = 0;
									if(isThumb)
									{
										w = 50;
										h = (int)(w * size.Height / size.Width);
									}
									var bi = Helpers.GetBitmapImageFromStream(st, w, h);
									Helpers.GetPngStream(bi, ms);
									ms.Seek(0, SeekOrigin.Begin);
									ms.CopyTo(stream);
								}
								return true;
							});
						}
						break;
					case "text/csv":
						{
							var fields = new string[] { category.CategoryID.ToString(), category.CategoryName, category.Description };
							Quote(fields);
							var str = string.Join(",", fields);
							str.WriteToStream(stream);
						}
						break;
					case "text/plain":
						{
							var str = string.Format("Category: {0} - Name: {1} - Description: {2}", category.CategoryID.ToString(), category.CategoryName, category.Description);
							str.WriteToStream(stream);
						}
						break;
					case "text/html":
						{
							var str = string.Format("<div class='itemCategory'>Id: {0} - Name: {1} - Description: {2}</div>", category.CategoryID.ToString(), category.CategoryName, category.Description);
							str.WriteToStream(stream);
						}
						break;
				}


				contentHeaders.Add("X-CategoryName", category.CategoryName);	// custom header!
			});
			t.Start();
			return t;
		}

		private void Quote(string[] fields)
		{
			for(int i = 0; i < fields.Length; i++)
			{
				fields[i] = fields[i].Replace("\"", "\"\"");
				fields[i] = "\"" + fields[i] + "\"";
			}
		}
	}



}