﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using System.Windows.Media.Imaging;
using System.Windows.Media;
using System.Threading;
using System.Windows;
using System.IO;

namespace AW3
{
	public static class Helpers
	{
		public static T FilenameToStream<T>(string fullPathName, Func<Stream, T> streamFunc)
		{
			if (string.IsNullOrEmpty(fullPathName) || !File.Exists(fullPathName))
				return default(T);
			using (FileStream st = new FileStream(fullPathName, FileMode.Open, FileAccess.Read, FileShare.None))
			{
				return streamFunc(st);
			}
		}

		public static T BlobToStream<T>(byte[] blob, Func<Stream, T> streamFunc)
		{
			return BlobToStream(blob, streamFunc, 0, blob.Length);
		}

		public static T BlobToStream<T>(byte[] blob, Func<Stream, T> streamFunc, int index, int count)
		{
			if (blob == null || blob.Length == 0)
				return default(T);
			using (MemoryStream st = new MemoryStream(blob, index, count))
			{
				return streamFunc(st);
			}
		}

		public static void WriteToStream(this string text, Stream stream)
		{
			if (string.IsNullOrEmpty(text))
				return;

			var blob = System.Text.Encoding.UTF8.GetBytes(text);
			stream.Write(blob, 0, blob.Length);
		}

		//public static void CopyTo(this Stream source, Stream target)
		//{
		//    if (source != null)
		//    {
		//        MemoryStream ms = source as MemoryStream;

		//        if (ms != null)
		//        {
		//            ms.WriteTo(target);
		//        }
		//        else
		//        {
		//            byte[] buffer = new byte[2048];
		//            int length = buffer.Length;
		//            int size;

		//            while ((size = source.Read(buffer, 0, length)) != 0)
		//            {
		//                target.Write(buffer, 0, size);
		//            }
		//        }
		//    }
		//}

		public static void SaveOnFile(this Stream stream, string filename)
		{
			var ms = stream as MemoryStream;
			if (ms != null)
			{
				File.WriteAllBytes(filename, ms.ToArray());
			}
			else
			{
				long OldPos = stream.Position;
				stream.Seek(0L, SeekOrigin.Begin);
				byte[] buffer = new byte[stream.Length];
				int read = stream.Read(buffer, 0, buffer.Length);
				if (read != buffer.Length)
					throw new Exception("SaveOnFile: la lunghezza dello stream e l'ultima lettura non corrispondono (length mismatch)");
				File.WriteAllBytes(filename, buffer);
			}
		}



		/// <summary>
		/// Caller will allocate and dispose the stream
		/// </summary>
		/// <param name="stream"></param>
		/// <returns></returns>
		public static BitmapImage GetBitmapImageFromStream(Stream stream)
		{
			return GetBitmapImageFromStream(stream, 0, 0);
		}

		public static BitmapImage GetBitmapImageFromStream(Stream stream, int width, int height)
		{
			var bitmap = new BitmapImage();
			bitmap.BeginInit();
			if(width > 0)
				bitmap.DecodePixelWidth = width;
			if(height > 0)
				bitmap.DecodePixelHeight = height;
			bitmap.StreamSource = stream;
			bitmap.CacheOption = BitmapCacheOption.OnLoad;
			bitmap.EndInit();

			bitmap.Freeze();
			return bitmap;
		}

		/// <summary>
		/// Caller will allocate and dispose both streams
		/// </summary>
		/// <param name="sourceStream"></param>
		/// <param name="pngStream"></param>
		public static void GetPngStream(Stream sourceStream, Stream pngStream)
		{
			if (sourceStream == null || pngStream == null)
				throw new ArgumentNullException();

			PngBitmapEncoder encoder = new PngBitmapEncoder();
			encoder.Interlace = PngInterlaceOption.Off;
			encoder.Frames.Add(BitmapFrame.Create(sourceStream));
			encoder.Save(pngStream);

			pngStream.Seek(0, SeekOrigin.Begin);
		}

		public static void GetPngStream(BitmapImage source, Stream pngStream)
		{
			if (source == null || pngStream == null)
				throw new ArgumentNullException();

			PngBitmapEncoder encoder = new PngBitmapEncoder();
			encoder.Interlace = PngInterlaceOption.Off;
			encoder.Frames.Add(BitmapFrame.Create(source));
			encoder.Save(pngStream);

			pngStream.Seek(0, SeekOrigin.Begin);
		}



		/*
				public static BitmapSource GetBitmapFromString(string str, string fontName, double fontSize)
				{
					FormattedText text = new FormattedText(str,
						Thread.CurrentThread.CurrentCulture,
						System.Windows.FlowDirection.LeftToRight,
						new Typeface(new FontFamily(fontName), FontStyles.Normal, FontWeights.Normal, FontStretches.Normal), fontSize, Brushes.Black);

					int imageWidth = (int)Math.Ceiling(text.Width);
					int imageHeight = (int)Math.Ceiling(text.Height);

					var drawingVisual = new DrawingVisual();
					DrawingContext drawingContext = drawingVisual.RenderOpen();
					drawingContext.DrawRectangle(Brushes.LightYellow, null, new Rect(0, 0, imageWidth, imageHeight));
					drawingContext.DrawText(text, new Point(0, 0));
					drawingContext.Close();

					var bitmap = new RenderTargetBitmap(imageWidth, imageHeight, 96, 96, PixelFormats.Pbgra32);
					bitmap.Render(drawingVisual);

					return bitmap;
				}

				/// <summary>
				/// Must be disposed from the caller
				/// </summary>
				/// <param name="str"></param>
				/// <returns></returns>
				public static Stream GetPngMemoryStream(string str)
				{
					BitmapSource bmp = GetBitmapFromString(str, "Arial", 48);

					MemoryStream stream = new MemoryStream();

					PngBitmapEncoder encoder = new PngBitmapEncoder();
					encoder.Interlace = PngInterlaceOption.Off;
					encoder.Frames.Add(BitmapFrame.Create(bmp));
					encoder.Save(stream);

					stream.Seek(0, SeekOrigin.Begin);
					return stream;
				}
				*/

		public static Size GetImageSizeCm(Stream stream)
		{
			if (stream == null)
				return new Size(0, 0);
			try
			{
				BitmapImage src = new BitmapImage();
				src.BeginInit();
				src.StreamSource = stream;
				src.CacheOption = BitmapCacheOption.OnLoad;
				src.CreateOptions = BitmapCreateOptions.None;
				src.EndInit();
				Size size = new Size(src.PixelWidth * 2.54 / src.DpiX, src.PixelHeight * 2.54 / src.DpiY);
				return size;
			}
			catch (Exception)
			{
				System.Diagnostics.Trace.WriteLine("Errore durante il tentativo di leggere le dimensioni dell'immagine");
				return new Size(0, 0);
			}
		}

	}
}