﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using System.Net.Http.Formatting;
using System.Net.Http.Headers;
using System.IO;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace AW3Client
{
	public class CategoryFormatter : MediaTypeFormatter
	{
		public CategoryFormatter()
		{
			this.SupportedMediaTypes.Add(new MediaTypeHeaderValue("text/csv"));
		}

		protected override bool CanReadType(Type type)
		{
			return true;
		}

		protected override System.Threading.Tasks.Task<object> OnReadFromStreamAsync(Type type, Stream stream, HttpContentHeaders contentHeaders, FormatterContext formatterContext)
		{
			var task = new Task<object>(() =>
				{
					if(type != typeof(Category))
						throw new ArgumentException();
					string csv = stream.GetString();

					List<string> parts = new List<string>();
					Regex re = new Regex(@"""[^""\r\n]*""|'[^'\r\n]*'|[^,\r\n]*");
					Match m = re.Match(csv);
					while(m.Success)
					{
						if(!string.IsNullOrEmpty(m.Value))
							parts.Add(m.Value.Trim('\"'));
						m = m.NextMatch();
					}

					if(parts.Count != 3)
						throw new ArgumentException();

					int id;
					if(!int.TryParse(parts[0], out id))
						throw new ArgumentException();

					Category category = new Category()
					{
						CategoryID = id,
						CategoryName = parts[1],
						Description = parts[2],
					};
					return category;
				});
			task.Start();
			return task;
		}

		protected override bool CanWriteType(Type type)
		{
			return true;
		}

		protected override Task OnWriteToStreamAsync(Type type, object value, Stream stream, HttpContentHeaders contentHeaders, FormatterContext formatterContext, System.Net.TransportContext transportContext)
		{
			var task = new Task(() =>
				{
					var category = value as Category;
					if(category == null)
						return;

					var fields = new string[] { category.CategoryID.ToString(), category.CategoryName, category.Description };
					Quote(fields);
					var str = string.Join(",", fields);
					str.WriteToStream(stream);
				});
			return task;
		}

		private void Quote(string[] fields)
		{
			for (int i = 0; i < fields.Length; i++)
			{
				fields[i] = fields[i].Replace("\"", "\"\"");
				fields[i] = "\"" + fields[i] + "\"";
			}
		}


	}
}